/**********************************************************************
Description:

  Integer function check_reg_uniformity.c
  Evaluates surface uniformity.
  Called from get_pxldat.c.

Input arguments:
  none

  Inputs from mask_processing_constants.h

Output arguments:
  none

  int uniform       value of 1 if uniform, otherwise 0
                    (returned through function call)

Revision History:
  05/2007  R. Frey  Original version.
  10/2012  R. Frey  Adapted to MVCCM software
  11/2015  R. Frey  Code clean-up.
  06/2021  R. Frey  Uses GEOS snow and ice instead of NISE.

***********************************************************************/

/* Includes */

#include <stdio.h>
//#include "mask_processing_constants.h"
//#include "granule.h"
//#include "pixel.h"
//#include "ancillary.h"
//#include "granule_geolocation.h"

void check_reg_uniformity(int eles, int line_edge, int elem_edge, int scan, int elem, unsigned char eco_type,
                          unsigned char *eco, float *snowfr, float *icefr, unsigned char *lst,
                          int *land, int *coast, int *water, int *loc_uniform)


{

/*  Declarations */

    //int loc_uniform;
    int nland;
    int nwater;
    int ncoast;
    int ntotal;
    int nl, ne;
    int imve, imvl;
    int pxsnow;

    const int nlcntx = 3;
    const int necntx = 3;
    long int j, ide;

    unsigned char lsf;

    float snow_frc, ice_frc;

/*  Initializations */

    nwater = 0;
    nland = 0;
    ncoast = 0;
    ntotal = nlcntx * necntx;
    *loc_uniform = 1;
    imve = (necntx-1) / 2;
    imvl = (nlcntx-1) / 2;

//  Check for edges of granule.
    if(line_edge == 1 || elem_edge == 1) {

      *loc_uniform = 0;

    }
    else {

//    Check region (3x3 pixel area centered on current pixel) for scene uniformity.

      for ( nl=0; nl<nlcntx; ++nl) {
        for ( ne=0; ne<necntx; ++ne) {

    	  j = ( (scan + (nl - imvl)) * eles) + elem;
    	  ide = j + (ne - imve);

//        Ecosystem consistency.
    	  if ( (eco_type - *(eco+ide)) != 0) *loc_uniform = 0;

//        Snow/ice consistency.
    	  snow_frc = *(snowfr+ide);
    	  ice_frc = *(icefr+ide);
    	  pxsnow = 0;
    	  if ( (snow_frc > 0.25 && snow_frc <= 1.0) || (ice_frc > 0.25) ) pxsnow = 1;
    	  if(pxsnow == 1) *loc_uniform = 0;

//        Count land, water, and coast pixels in region for consistency calculation
//        below.
    	  lsf = *(lst+ide);
    	  if(lsf == 1 || lsf == 4) {
            nland++;
    	  }
    	  else if(lsf == 2 || lsf == 3) {
    	    ncoast++;
    	  }
    	  else {
    	    nwater++;
    	  }

        }
      }

//    Check for coastlines in current region.
      if( (nwater + ncoast) == ntotal) {

        if(nwater != ntotal) {
          *loc_uniform = 0;
//        Provide "double coastlines".
          *coast = 1;
          *land = 1;
          *water = 0;
        }

      }
      else if( (nland + ncoast) == ntotal) {

        if(nland != ntotal) *loc_uniform = 0;

      }
      else {

        *loc_uniform = 0;
        *coast = 1;
        *land = 1;
        *water = 0;

      }

    }
}
