/*******************************************************************************
Description:

  Integer function get_NDVI_background.c
  Opens NDVI background file and reads the data.
  Called from collect_inputs.c.

Input arguments:
  Ancillary data directory
  Input NDVI file name

Output arguments:
  none

Function output:
  int return_code  successful completion is zero, otherwise non-zero

  Output parameters in stored in ancillary.h.

Revision History:
  10/2012     R. Frey   Original version

Calls:
  none

*******************************************************************************/

// Includes

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//HDF includes
#include <hdfi.h>
#include <hdf.h>
#include <mfhdf.h>
#include <HdfEosDef.h>

//MVCCM includes
//#include "granule_geolocation.h"
//#include "granule.h"
//#include "ancillary.h"

/******************************************************************************/

void get_NDVI_background(float *lat, float *lon, int res, char *anc_dir, char *NDVI_file, float *g_ndvi_background)

{

/******************************************************************************/

// Declarations and initializations.

/******************************************************************************/

   char NDVI_file_wpath[256];
   char NDVI_data[23][80];
   char SDS_name[10];
   char * ndvi_sub="/NDVI/";
   float map_res, wm_lon, nm_lat;
   //float *g_ndvi_background;
   long int nncols = 21600;
   long int nnrows = 10800;
   long int xindex, yindex, indx, idx;
   int32 start[2], stride[2], edge[2], sds_index, sds_id, irt, id;
   int jd, xerr, yerr, row, col, numberRows;
   int ii, jj;
   int msgcounta = 0, msgcounta_lim = 100;
   int msgcountb = 0, msgcountb_lim = 100;
   int16 *ndvi_arr;
   int fnday[23] = {1, 17, 33, 49, 65, 81, 97, 113, 129, 145, 161, 177, 193, 209, 225, 241, 257, 273, 289, 305, 321, 337, 353};

   const int eles=3200*res;
   const int lines=3232*res;

   int return_code = 0;

/******************************************************************************/

// Construct file name.

   (void) strcpy(NDVI_file_wpath, anc_dir);
   (void) strcat(NDVI_file_wpath, "/");
   (void) strcat(NDVI_file_wpath, NDVI_file);
// printf("NDVI file: %s\n", NDVI_file_wpath);

/******************************************************************************/

// Check filename here.

/******************************************************************************/

// Open file. */
   id = SDstart(NDVI_file_wpath, DFACC_READ);
   if (id <= 0) {
     printf("Could not open NDVI file %s\n",NDVI_file_wpath);
     //return (-1);
   }

// Select proper SDS and read the data */

   start[0] = 0;
   start[1] = 0;
   stride[0] = 1;
   stride[1] = 1;
   edge[1] = nncols;
   edge[0] = nnrows;
   (void) strcpy(SDS_name, "NDVI");

   ndvi_arr = (int16 *) malloc(21600 * 10800 * 2);
   sds_index = SDnametoindex(id, SDS_name);
   sds_id = SDselect(id, sds_index);
   irt = SDreaddata(sds_id, start, stride, edge, ndvi_arr);
   if (irt < 0) {
     printf("Could not read NDVI file %s\n",NDVI_file_wpath);
     //return (-2);
   }

// Close NDVI file. */
   id = SDend(id);

   map_res = 360.0 / (float)(nncols);
   wm_lon = (map_res / 2.0) - 180.0;
   nm_lat = 90.0 - (map_res / 2.0);

// Loop through current data and assign NDVI value for each pixel. */

   //g_ndvi_background = (float *)malloc(eles*lines*sizeof(float));

   idx = 0;
   numberRows = lines;
   for ( row=0;row<(numberRows );++row) {
     for ( col=0;col<eles;++col) {

//     Initialize.
       xerr = 0;
       yerr = 0;
       g_ndvi_background[idx] = 32.767;

       xindex = (long int)((lon[idx] - wm_lon) / map_res);
       yindex = (long int)((nm_lat - lat[idx]) / map_res);

       if(xindex < 0 || xindex> nncols) {
         xerr = 1;
         if(msgcounta <= msgcounta_lim){
           printf("xindex out of bounds on NDVI map! \n");
           msgcounta += 1;
         }
         return_code = 1;
       }
       if(yindex < 0 || yindex> nnrows) {
         yerr = 1;
         if(msgcountb <= msgcountb_lim){
           printf("yindex out of bounds on NDVI map! \n");
           msgcountb += 1;
         }
         return_code = 2;
       }

       if(xerr == 0 && yerr == 0) {
         indx = (yindex*nncols) + xindex;
         g_ndvi_background[idx] = *(ndvi_arr + indx) * 0.001;

//       if (idx < 10) {
//         printf("ndvi %d %d %ld %ld %ld %f %f %d %f\n", row, col, xindex, yindex, indx, grn_geo.lat[idx], grn_geo.lon[idx], *(ndvi_arr + indx), g_ndvi_background[idx]);
//       }

       }

       idx++;

     }
   }

   //grn_anc.ndvibk = g_ndvi_background;

   free (ndvi_arr);

/******************************************************************************/

//   return (return_code);

}
