/*******************************************************************************
Description:

  integer function get_Olson_eco.c
  Opens and reads Olson ecosystem file.
  Called from collect_inputs.c.

Input arguments:
  anc_dir               ancillary data directory

Output arguments:
  none

Function output:
  int return_code       successful completion is zero, otherwise non-zero

  Input file name (static) defined here.

  Output stored in header file ancillary.h.

Revision History:
  10/2012     R. Frey   Original version

Calls:
  none

*******************************************************************************/

// Includes

/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//MVCCM includes
//#include "granule_geolocation.h"
//#include "granule.h"
//#include "ancillary.h"

/******************************************************************************/

int get_Olson_eco(float *lat, float *lon, int res, char *anc_dir, unsigned char *granule_ecotype)

{

/******************************************************************************/

//  Declarations and initializations.
//
/******************************************************************************/

    extern long int lround(double);
    extern void getcoord_(double*, double*, double*, double*);

    char eco_file[256];
    unsigned char eco_index;
    //unsigned char *granule_ecotype;
    double lt, ln, ltt, lnn;
    double drow;
    double dcol;
    long int idx, irow, icol, byte_loc;
    long int max_row = 17347;
    long int max_col = 40031;
    int row, col, numberRows;

    const int eles=3200*res;
    const int lines=3232*res;

    size_t bytes_read;
    FILE *eco_file_ptr;

//  Define static ecosystem file name.
    char eco_file_name[] = "goge1_2_img.v1";

    int return_code = 0;

/******************************************************************************/

/*  Get file name */
    (void) strcpy(eco_file, anc_dir);
    (void) strcat(eco_file, "/");
    (void) strcat(eco_file, eco_file_name);
//  printf("ecosystem file: %s \n", eco_file);

/*  Open file */
    eco_file_ptr = fopen(eco_file, "r");
    if(eco_file_ptr == NULL) {
      printf("Cannot open ecosystem file %s \n", eco_file);
      //return (-1);
    }

    //granule_ecotype = (unsigned char *)malloc(eles*lines*sizeof(unsigned char));
    numberRows = lines;
    idx = 0;
    for ( row=0; row<(numberRows); ++row) {
      for ( col=0; col<eles ;++col) {

//      Get lat/lon
        lt = (double) lat[idx];
        ln = (double) lon[idx];
        ltt = lt;
        lnn = ln;

//      Get row/col of data grid corresponding to lat/lon.
        (void) getcoord_(&lt, &ln, &drow, &dcol);

        irow = lround(drow);
        if(irow > max_row) irow = max_row;
        if(irow < 1) irow = 1;
        icol = lround(dcol);
        if(icol > max_col) icol = max_col;
        if(icol < 1) icol = 1;

        byte_loc = ( ((irow-1) * max_col) + icol) - 1;

//      Read file
        fseek(eco_file_ptr, byte_loc, SEEK_SET);
        bytes_read = fread((char *) &eco_index, sizeof(eco_index), 1, eco_file_ptr);

        granule_ecotype[idx] = eco_index;

//      if(idx < 10)
//        printf("Olson ecosystem index: %d %d\n", idx, eco_index);

         ++idx;

      }
    }

/*  Close file */
    (void) fclose(eco_file_ptr);

    //grn_anc.eco = granule_ecotype;

/******************************************************************************/

    //return (return_code);

}
