/*******************************************************************************
Description:

  Integer function get_ti_vars_geos.c
  Calculates time-interpolated variables from the two GEOS times.
  Called from get_GEOS.c

Input arguments:
  wt               time interpolation weight for later GEOS time

Output arguments:
  none

Function output:
  int return_code  successful completion is zero, otherwise non-zero

  Inputs are taken from pointers stored in ancillary.h.
  Pointers to time-interpolated variables are stored in ancillary.h.

Revision History:
  10/2012 R. Frey  Original version

Calls:
  none

*******************************************************************************/

// Includes
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//MVCCM includes
#include "ancillary.h"

/******************************************************************************/

int get_ti_vars(float wt)

{

/******************************************************************************/

//  Declarations and initializations.

    long int n;
    int return_code = 0;

/******************************************************************************/

//  Allocate memory for time-interpolated data.
    if ((ti_sfct = (float *) malloc(sizeof(float) * nwp_cells)) == NULL) {
      printf("Could not allocate memory for ti_sfct\n");
      return(-2);
    }
    if ((ti_tpw = (float *) malloc(sizeof(float) * nwp_cells)) == NULL) {
      printf("Could not allocate memory for ti_tpw\n");
      return(-4);
    }

/******************************************************************************/

    for (n=0; n<nwp_cells; n++) {

      ti_sfct[n] = (g_sfct1[n] >= 0.0 && g_sfct2[n] >= 0.0) ?
           (wt*(g_sfct2[n]) + (1.0-wt)*(g_sfct1[n])) : -999.99;
//    if(n >= 1000 && n <= 1009)
//      printf("ti sfc t: %ld %f %f %f %f\n", n, wt, g_sfct1[n], g_sfct2[n], ti_sfct[n]);

      ti_tpw[n] = (g_tpw1[n] >= 0.0 && g_tpw2[n] >= 0.0) ?
           (wt*(g_tpw2[n]/10.0) + (1.0-wt)*(g_tpw1[n]/10.0)) : -999.99;
//    if(n >= 1000 && n <= 1009)
//      printf("ti tpw: %ld %f %f %f %f\n", n, wt, g_tpw1[n], g_tpw2[n], ti_tpw[n]);

    }

/******************************************************************************/

//  Free unneeded memory.
    free(g_sfct1);
    free(g_sfct2);
    free(g_tpw1);
    free(g_tpw2);

/******************************************************************************/

    return (return_code);

/******************************************************************************/

}
